from airflow.models import BaseOperator

import logging

from typing import TYPE_CHECKING

from kensu.utils.kensu_provider import KensuProvider

if TYPE_CHECKING:
    from airflow.utils.context import Context

KENSU_ERRORS_FAIL_JOB = True
COLLECTOR_STATUS_INIT = "initializing..."
COLLECTOR_STATUS_DONE = "done."


def airflow_init_kensu(
        airflow_operator=None,  # type: BaseOperator
        project_name=None,
        process_name=None,
):
    if airflow_operator is not None:
        if project_name is None:
            from airflow.models import Variable
            project_name = Variable.get("KSU_PROJECT_NAME", default_var='Airflow :: ' + airflow_operator.dag_id)
        if process_name is None:
            process_name = airflow_operator.task_id

    # it seems GCP Composer's env variables are actually not passed (at least to some of the Airflow Operators)
    # so we set the important/sensitive settings explicitly from Airflow (secured) variables
    from airflow.models import Variable
    api_url = Variable.get("KSU_KENSU_INGESTION_URL", default_var=None)
    auth_token = Variable.get("KSU_KENSU_INGESTION_TOKEN", default_var=None)

    KensuProvider().initKensu(
        kensu_ingestion_url=api_url,
        kensu_ingestion_token=auth_token,
        init_context=True,
        project_name=project_name,
        process_name=process_name,
        bigquery_support=True)


def full_gs_uri(bucket, obj_path):
    return f"gs://{bucket}/{obj_path}"


def handle_ex(obj, ex):
    logging.warning(f"Kensu collector failed for {type(obj)}", ex)
    if KENSU_ERRORS_FAIL_JOB:
        raise ex


def log_status(obj, status):
    logging.info(f"Kensu collector for {type(obj)} is {status}")



# coding: utf-8

"""
    

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: beta
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat

from six import iteritems


class ModelMetrics(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'pk': 'ModelMetricsPK',
        'metrics': 'dict(str, object)',
        'other_metrics_as_json': 'str',
        'hyper_params_as_json': 'str'
    }

    attribute_map = {
        'pk': 'pk',
        'metrics': 'metrics',
        'other_metrics_as_json': 'otherMetricsAsJson',
        'hyper_params_as_json': 'hyperParamsAsJson'
    }

    def __init__(self, pk=None, metrics=None, other_metrics_as_json=None, hyper_params_as_json=None):
        """
        ModelMetrics - a model defined in Swagger
        """

        self._pk = None
        self._metrics = None
        self._other_metrics_as_json = None
        self._hyper_params_as_json = None

        self.pk = pk
        self.metrics = metrics
        if other_metrics_as_json is not None:
          self.other_metrics_as_json = other_metrics_as_json
        if hyper_params_as_json is not None:
          self.hyper_params_as_json = hyper_params_as_json

    @property
    def pk(self):
        """
        Gets the pk of this ModelMetrics.

        :return: The pk of this ModelMetrics.
        :rtype: ModelMetricsPK
        """
        return self._pk

    @pk.setter
    def pk(self, pk):
        """
        Sets the pk of this ModelMetrics.

        :param pk: The pk of this ModelMetrics.
        :type: ModelMetricsPK
        """
        if pk is None:
            raise ValueError("Invalid value for `pk`, must not be `None`")

        self._pk = pk

    @property
    def metrics(self):
        """
        Gets the metrics of this ModelMetrics.
        Metrics of the trained model as a map between metric names and their value as double

        :return: The metrics of this ModelMetrics.
        :rtype: dict(str, object)
        """
        return self._metrics

    @metrics.setter
    def metrics(self, metrics):
        """
        Sets the metrics of this ModelMetrics.
        Metrics of the trained model as a map between metric names and their value as double

        :param metrics: The metrics of this ModelMetrics.
        :type: dict(str, object)
        """
        if metrics is None:
            raise ValueError("Invalid value for `metrics`, must not be `None`")

        self._metrics = metrics

    @property
    def other_metrics_as_json(self):
        """
        Gets the other_metrics_as_json of this ModelMetrics.
        If relevant, extra non double metrics can be added as a valid JSON string

        :return: The other_metrics_as_json of this ModelMetrics.
        :rtype: str
        """
        return self._other_metrics_as_json

    @other_metrics_as_json.setter
    def other_metrics_as_json(self, other_metrics_as_json):
        """
        Sets the other_metrics_as_json of this ModelMetrics.
        If relevant, extra non double metrics can be added as a valid JSON string

        :param other_metrics_as_json: The other_metrics_as_json of this ModelMetrics.
        :type: str
        """

        self._other_metrics_as_json = other_metrics_as_json

    @property
    def hyper_params_as_json(self):
        """
        Gets the hyper_params_as_json of this ModelMetrics.

        :return: The hyper_params_as_json of this ModelMetrics.
        :rtype: str
        """
        return self._hyper_params_as_json

    @hyper_params_as_json.setter
    def hyper_params_as_json(self, hyper_params_as_json):
        """
        Sets the hyper_params_as_json of this ModelMetrics.

        :param hyper_params_as_json: The hyper_params_as_json of this ModelMetrics.
        :type: str
        """

        self._hyper_params_as_json = hyper_params_as_json

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, ModelMetrics):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



from kensu.pandas import DataFrame,Series
from kensu.utils.kensu_provider import KensuProvider
import sklearn.linear_model as lm
from kensu.utils.helpers import eventually_report_in_mem
from kensu.numpy import ndarray
from kensu.client.models import Model,ModelPK,ModelRef,ModelTraining,ModelTrainingPK,ModelTrainingRef
from kensu.utils.wrappers import remove_ksu_wrapper

class LogisticRegression(lm.LogisticRegression):

    def fit(self, X, y, sample_weight=None):
        X_train = X.get_df()
        if isinstance(y,DataFrame):
            y_train = y.get_df()
        elif isinstance(y,Series):
            y_train = y.get_s()
        result = super(LogisticRegression,self).fit( X_train , y_train,sample_weight )
        kensu = KensuProvider().instance()

        metrics = kensu.extractors.extract_machine_learning_metrics(result,x_train = X_train, y_train = y_train,classification = True )
        kensu = KensuProvider().instance()

        train_X_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(X_train, kensu.default_physical_location_ref))
        train_X_sc = eventually_report_in_mem(kensu.extractors.extract_schema(train_X_ds, X_train))

        train_y_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(y, kensu.default_physical_location_ref))
        train_y_sc = eventually_report_in_mem(kensu.extractors.extract_schema(train_y_ds, y))

        result_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(result, kensu.default_physical_location_ref,format='Sklearn Model'))
        result_sc = eventually_report_in_mem(kensu.extractors.extract_schema(result_ds, result))

        hyperparams = result.get_params()

        if kensu.mapping:
            for result_col in [s.name for s in result_sc.pk.fields]:
                for col in [s.name for s in train_X_sc.pk.fields]:
                    kensu.add_dependencies_mapping(result_sc.to_guid(),result_col,train_X_sc.to_guid(),col,'Model Train')
                for col in [s.name for s in train_y_sc.pk.fields]:
                    kensu.add_dependencies_mapping(result_sc.to_guid(), result_col, train_y_sc.to_guid(), col,
                                                 'Model Train')

        attr = [result,'SkLearn.LogisticRegression',metrics,hyperparams,result_ds,result_sc]
        self.attr = attr

        return result

    def predict(self, X):
        if 'kensu' in str(X.__class__):
            X_test = X.get_df()
        else:
            X_test = X

        result = super(LogisticRegression, self).predict(X_test)
        kensu = KensuProvider().instance()

        X_test_ds = eventually_report_in_mem(
            kensu.extractors.extract_data_source(X_test, kensu.default_physical_location_ref))
        X_test_sc = eventually_report_in_mem(kensu.extractors.extract_schema(X_test_ds, X_test))

        ksu_result = ndarray.using(result)

        result_ds = eventually_report_in_mem(
            kensu.extractors.extract_data_source(ksu_result, kensu.default_physical_location_ref))
        result_sc = eventually_report_in_mem(kensu.extractors.extract_schema(result_ds, ksu_result))

        result = ksu_result

        try:
            model_ds = self.attr[4]
            model_sc = self.attr[5]
        except:
            model_ds = None
            model_sc = None

        if model_sc is not None:
            model = eventually_report_in_mem(model_ds)
            model_sc = eventually_report_in_mem(model_sc)
        if kensu.mapping:
            for result_col in [s.name for s in result_sc.pk.fields]:
                for col in [s.name for s in X_test_sc.pk.fields]:
                    kensu.add_dependencies_mapping(result_sc.to_guid(), result_col, X_test_sc.to_guid(), col,
                                                 'Model Predict')
                if model_sc is not None:
                    for col in [s.name for s in model_sc.pk.fields]:
                        kensu.add_dependencies_mapping(result_sc.to_guid(), result_col, model_sc.to_guid(), col,
                                                     'Model Predict')
        return result


class LinearRegression(lm.LinearRegression):
    def fit(self, X, y, sample_weight=None):

        X_train=remove_ksu_wrapper(X)
        y_train = remove_ksu_wrapper(y)

        result = super(LinearRegression, self).fit(X_train, y_train, sample_weight)
        kensu = KensuProvider().instance()

        metrics = kensu.extractors.extract_machine_learning_metrics(result, x_train=X_train, y_train=y_train)
        kensu = KensuProvider().instance()

        train_X_ds = eventually_report_in_mem(
            kensu.extractors.extract_data_source(X_train, kensu.default_physical_location_ref))
        train_X_sc = eventually_report_in_mem(kensu.extractors.extract_schema(train_X_ds, X_train))

        train_y_ds = eventually_report_in_mem(
            kensu.extractors.extract_data_source(y, kensu.default_physical_location_ref))
        train_y_sc = eventually_report_in_mem(kensu.extractors.extract_schema(train_y_ds, y))

        result_ds = eventually_report_in_mem(
            kensu.extractors.extract_data_source(result, kensu.default_physical_location_ref, format='Sklearn Model'))
        result_sc = eventually_report_in_mem(kensu.extractors.extract_schema(result_ds, result))

        hyperparams = result.get_params()

        if kensu.mapping:
            for result_col in [s.name for s in result_sc.pk.fields]:
                for col in [s.name for s in train_X_sc.pk.fields]:
                    kensu.add_dependencies_mapping(result_sc.to_guid(), result_col, train_X_sc.to_guid(), col,
                                                   'Model Train')
                for col in [s.name for s in train_y_sc.pk.fields]:
                    kensu.add_dependencies_mapping(result_sc.to_guid(), result_col, train_y_sc.to_guid(), col,
                                                   'Model Train')

        attr = [result, 'SkLearn.LinearRegression', metrics, hyperparams, result_ds, result_sc]
        self.attr = attr

        return result

    def predict(self, X):
        if 'kensu' in str(X.__class__):
            X_test = X.get_df()
        else:
            X_test = X

        result = super(LinearRegression, self).predict(X_test)
        kensu = KensuProvider().instance()

        X_test_ds = eventually_report_in_mem(
            kensu.extractors.extract_data_source(X_test, kensu.default_physical_location_ref))
        X_test_sc = eventually_report_in_mem(kensu.extractors.extract_schema(X_test_ds, X_test))

        ksu_result = ndarray.using(result)

        result_ds = eventually_report_in_mem(
            kensu.extractors.extract_data_source(ksu_result, kensu.default_physical_location_ref))
        result_sc = eventually_report_in_mem(kensu.extractors.extract_schema(result_ds, ksu_result))

        result = ksu_result

        try:
            model_ds = self.attr[4]
            model_sc = self.attr[5]
        except:
            model_ds = None
            model_sc = None

        if model_sc is not None:
            model = eventually_report_in_mem(model_ds)
            model_sc = eventually_report_in_mem(model_sc)
        if kensu.mapping:
            for result_col in [s.name for s in result_sc.pk.fields]:
                for col in [s.name for s in X_test_sc.pk.fields]:
                    kensu.add_dependencies_mapping(result_sc.to_guid(), result_col, X_test_sc.to_guid(), col,
                                                   'Model Predict')
                if model_sc is not None:
                    for col in [s.name for s in model_sc.pk.fields]:
                        kensu.add_dependencies_mapping(result_sc.to_guid(), result_col, model_sc.to_guid(), col,
                                                       'Model Predict')
        return result


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2021 Kensu Inc
#
import logging
import re
from typing import Union

from google.cloud.bigquery import Table, TableReference

from kensu.google.cloud.bigquery.job.bq_helpers import BqKensuHelpers
from kensu.utils.dsl.extractors.external_lineage_dtos import KensuDatasourceAndSchema, GenericComputedInMemDs, \
    ExtDependencyEntry
from kensu.utils.helpers import extract_ksu_ds_schema
from kensu.utils.kensu import Kensu
import google.cloud.bigquery as bq
import sqlparse

logger = logging.getLogger(__name__)


class BqOfflineParser:

    @staticmethod
    def normalize_table_refs(q):
        # FIXME: this might be still quite error prone in non-standard use-cases...
        """
        >>> normalize_table_refs('SELECT * FROM `a1`.`b2`.`c3`')
        'SELECT * FROM `a1.b2.c3`'
        """
        matches = re.findall(r'`([a-zA-Z0-9-_]+)`.`([a-zA-Z0-9-_]+)`.`([a-zA-Z0-9-_]+)`', q)
        for (s1, s2, s3) in matches:
            q = q.replace(f'`{s1}`.`{s2}`.`{s3}`', f'`{s1}.{s2}.{s3}`')

        return q

    @staticmethod
    def get_referenced_tables_metadata(
            kensu: Kensu,
            client: bq.Client,
            job,  # type: google.cloud.bigquery.job.QueryJob
            query: str = None,
            table: Table = None):
        table_infos = []
        if job:
            # Referenced tables for the job. Queries that reference more than 50 tables will not have a complete list.
            # FIXME: separate referenced from ddl target
            referenced_tables = job.referenced_tables + [j for j in [job.ddl_target_table] if j] # type: list[TableReference]
            # [TableReference(DatasetReference('project', 'db'), 'table')]
            table_infos = list([BqOfflineParser.table_ref_to_kensu(client, table_id=t)
                            for t in referenced_tables])
        is_bigquery_api_limit_reached = len(table_infos) > 49
        if not table_infos or is_bigquery_api_limit_reached:
            fallback_tables = list(BqOfflineParser.fallback_referenced_tables_from_sql(kensu=kensu,
                                                                              client=client,
                                                                              query=query,
                                                                              table=query))
            table_infos = list((table_infos + fallback_tables))
            #remove duplicates
            res = []
            for i in table_infos:
                if i not in res:
                    res.append(i)
        else:
            res=table_infos
        return BqOfflineParser.to_sql_util_metadata(res)


    # FIXME: or should we better simply fetch schema for ALL visible tables and databases !!!!???
    @staticmethod
    def fallback_referenced_tables_from_sql(
            kensu: Kensu,
            client: bq.Client,
            query: str = None,
            table: Table = None):
        # FIXME: return table/tableRef here?!
        if query:
            table_infos = BqOfflineParser.get_table_info_from_sql(client, query)
        elif table:
            table_infos = [BqOfflineParser.table_ref_to_kensu(client, table_id=table)]
        else:
            table_infos = []
        return table_infos


    @staticmethod
    def to_sql_util_metadata(table_infos):
        table_id_to_bqtable = {}
        metadata = {"tables": []}
        for table, ds, sc in table_infos:
            table_id = "`" + table.full_table_id.replace(":", ".") + "`"  # FIXME: replace this in DS extractor too!
            table_md = {
                "id": table_id,
                "schema": {
                    "fields": [{"name": f.name, "type": f.field_type} for f in sc.pk.fields]
                }
            }
            table_id_to_bqtable[table_id] = table
            metadata["tables"].append(table_md)
        return metadata,  table_id_to_bqtable, table_infos


    @staticmethod
    def table_ref_to_kensu(
            client: bq.Client,
            table_id: Union[Table, TableReference, str],
    ):
        table = client.get_table(table_id)
        ds, sc = BqKensuHelpers.table_to_kensu(table)
        return table, ds, sc

    @staticmethod
    def get_table_info_for_id(client: bq.Client, id: sqlparse.sql.Identifier or sqlparse.sql.Token):
        try:
            if isinstance(id,sqlparse.sql.Identifier):
                name = (id.get_real_name()).strip('`')
            elif isinstance(id,sqlparse.sql.Token):
                name = id.value.strip('`')
            return BqOfflineParser.table_ref_to_kensu(client, table_id=name)
        except Exception as e:
            logger.debug("get_table_info_for_id failed for table={}, maybe not BQ table: {}".format(id, str(e)))
            # FIXME this is because the current find_sql_identifiers also returns the column names...
            #  (see aboveREF_GET_TABLE)
            #  Therefore get_table of a column name should fail
            return None

    @staticmethod
    def get_table_info_from_sql(client: bq.Client, query: str):
        sq = sqlparse.parse(query)
        ids = BqOfflineParser.find_sql_identifiers(sq[0].tokens)  # FIXME we only take the first element
        table_infos = list(
            filter(lambda x: x is not None, [BqOfflineParser.get_table_info_for_id(client, id) for id in ids]))
        return table_infos

    @staticmethod
    def find_sql_identifiers(tokens):
        for e in tokens:
            for t in e.flatten():
                if str(t.ttype) == "Token.Name":
                    yield t

    @staticmethod
    def fallback_lineage(kensu, table_infos, dest):
        res_ds, res_schema = extract_ksu_ds_schema(kensu, orig_variable=dest, report=False, register_orig_data=False)
        res_field_names = [f.name for f in res_schema.pk.fields]
        all_inputs = []
        for input_table, input_ds, input_sc in table_infos:
            if res_ds.pk.location == input_ds.pk.location:
                continue
            input = KensuDatasourceAndSchema.for_path_with_opt_schema(
                kensu,
                ds_path=input_ds.pk.location,
                ds_name=input_ds.name,
                format='BigQuery table',
                categories=None,
                maybe_schema=[(f.name, f.field_type) for f in input_sc.pk.fields],
                # FIXME: hmm, are stats not implemented for fallback mode? also needs input filters
                f_get_stats=lambda: None
            )
            all_inputs.append(input)
        return GenericComputedInMemDs.for_direct_or_full_mapping(all_inputs=all_inputs, out_field_names=res_field_names)


# coding: utf-8

"""
    

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: beta
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# python 2 and python 3 compatibility library
from six import iteritems

from ..api_client import ApiClient
from ..configuration import Configuration


class KensuEntitiesApi(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        config = Configuration()
        if api_client:
            self.api_client = api_client
        else:
            if not config.api_client:
                config.api_client = ApiClient()
            self.api_client = config.api_client


    def report_code_base(self, body, **kwargs):
        """
        Report a new CodeBase
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_code_base(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param CodeBase body: A CodeVersion to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_code_base_with_http_info(body, **kwargs)
        else:
            (data) = self.report_code_base_with_http_info(body, **kwargs)
            return data

    def report_code_base_with_http_info(self, body, **kwargs):
        """
        Report a new CodeBase
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_code_base_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param CodeBase body: A CodeVersion to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_code_base" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_code_base`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/code-base', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_code_version(self, body, **kwargs):
        """
        Report a new CodeVersion
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_code_version(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param CodeVersion body: A CodeVersion to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_code_version_with_http_info(body, **kwargs)
        else:
            (data) = self.report_code_version_with_http_info(body, **kwargs)
            return data

    def report_code_version_with_http_info(self, body, **kwargs):
        """
        Report a new CodeVersion
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_code_version_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param CodeVersion body: A CodeVersion to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_code_version" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_code_version`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/codeVersion', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_data_source(self, body, **kwargs):
        """
        Report a new Datasource
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_data_source(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param DataSource body: A datasource to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_data_source_with_http_info(body, **kwargs)
        else:
            (data) = self.report_data_source_with_http_info(body, **kwargs)
            return data

    def report_data_source_with_http_info(self, body, **kwargs):
        """
        Report a new Datasource
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_data_source_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param DataSource body: A datasource to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_data_source" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_data_source`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/datasources', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_data_stats(self, body, **kwargs):
        """
        Report a new DataStats
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_data_stats(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param DataStats body: A DataStats to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_data_stats_with_http_info(body, **kwargs)
        else:
            (data) = self.report_data_stats_with_http_info(body, **kwargs)
            return data

    def report_data_stats_with_http_info(self, body, **kwargs):
        """
        Report a new DataStats
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_data_stats_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param DataStats body: A DataStats to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_data_stats" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_data_stats`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/dataStats', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_entity_batch(self, body, **kwargs):
        """
        Report new Batch of entities
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_entity_batch(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param BatchEntityReport body: Definition of entities to be reported (required)
        :return: BatchEntityReportResult
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_entity_batch_with_http_info(body, **kwargs)
        else:
            (data) = self.report_entity_batch_with_http_info(body, **kwargs)
            return data

    def report_entity_batch_with_http_info(self, body, **kwargs):
        """
        Report new Batch of entities
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_entity_batch_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param BatchEntityReport body: Definition of entities to be reported (required)
        :return: BatchEntityReportResult
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_entity_batch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_entity_batch`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities-batch', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_lineage_run(self, body, **kwargs):
        """
        Report new LineageRun
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_lineage_run(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param LineageRun body: A connection between a lineage and a process run to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_lineage_run_with_http_info(body, **kwargs)
        else:
            (data) = self.report_lineage_run_with_http_info(body, **kwargs)
            return data

    def report_lineage_run_with_http_info(self, body, **kwargs):
        """
        Report new LineageRun
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_lineage_run_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param LineageRun body: A connection between a lineage and a process run to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_lineage_run" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_lineage_run`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/lineage-run', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_model(self, body, **kwargs):
        """
        Report a new Model
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_model(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Model body: A Model to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_model_with_http_info(body, **kwargs)
        else:
            (data) = self.report_model_with_http_info(body, **kwargs)
            return data

    def report_model_with_http_info(self, body, **kwargs):
        """
        Report a new Model
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_model_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Model body: A Model to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_model" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_model`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/model', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_model_metrics(self, body, **kwargs):
        """
        Report a new ModelMetrics
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_model_metrics(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ModelMetrics body: A ModelMetrics to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_model_metrics_with_http_info(body, **kwargs)
        else:
            (data) = self.report_model_metrics_with_http_info(body, **kwargs)
            return data

    def report_model_metrics_with_http_info(self, body, **kwargs):
        """
        Report a new ModelMetrics
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_model_metrics_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ModelMetrics body: A ModelMetrics to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_model_metrics" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_model_metrics`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/modelMetrics', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)


    def report_model_training(self, body, **kwargs):
        """
        Report a new ModelTraining
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_model_training(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ModelTraining body: A ModelTraining to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_model_training_with_http_info(body, **kwargs)
        else:
            (data) = self.report_model_training_with_http_info(body, **kwargs)
            return data

    def report_model_training_with_http_info(self, body, **kwargs):
        """
        Report a new ModelTraining
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_model_training_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ModelTraining body: A ModelTraining to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_model_training" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_model_training`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/modelTraining', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_physical_location(self, body, **kwargs):
        """
        Report a new Physical Location
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_physical_location(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param PhysicalLocation body: A physical location to be reported (i.e. a country and a city) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_physical_location_with_http_info(body, **kwargs)
        else:
            (data) = self.report_physical_location_with_http_info(body, **kwargs)
            return data

    def report_physical_location_with_http_info(self, body, **kwargs):
        """
        Report a new Physical Location
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_physical_location_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param PhysicalLocation body: A physical location to be reported (i.e. a country and a city) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_physical_location" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_physical_location`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/physical-location', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_process(self, body, **kwargs):
        """
        Report new Process
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Process body: Definition of process to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_process_with_http_info(body, **kwargs)
        else:
            (data) = self.report_process_with_http_info(body, **kwargs)
            return data

    def report_process_with_http_info(self, body, **kwargs):
        """
        Report new Process
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Process body: Definition of process to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_process" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_process`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/process', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_process_lineage(self, body, **kwargs):
        """
        Report new ProcessLineage
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process_lineage(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ProcessLineage body: A lineage of a process to be reported (between input and output schemas) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_process_lineage_with_http_info(body, **kwargs)
        else:
            (data) = self.report_process_lineage_with_http_info(body, **kwargs)
            return data

    def report_process_lineage_with_http_info(self, body, **kwargs):
        """
        Report new ProcessLineage
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process_lineage_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ProcessLineage body: A lineage of a process to be reported (between input and output schemas) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_process_lineage" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_process_lineage`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/process-lineage', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_process_run(self, body, **kwargs):
        """
        Report new Process run
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process_run(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ProcessRun body: Definition of process run to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_process_run_with_http_info(body, **kwargs)
        else:
            (data) = self.report_process_run_with_http_info(body, **kwargs)
            return data

    def report_process_run_with_http_info(self, body, **kwargs):
        """
        Report new Process run
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process_run_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ProcessRun body: Definition of process run to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_process_run" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_process_run`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/process-runs', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_process_run_stats(self, body, **kwargs):
        """
        Report new ProcessStats
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process_run_stats(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ProcessRunStats body: Process stats (e.g. number of subjects processed) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_process_run_stats_with_http_info(body, **kwargs)
        else:
            (data) = self.report_process_run_stats_with_http_info(body, **kwargs)
            return data

    def report_process_run_stats_with_http_info(self, body, **kwargs):
        """
        Report new ProcessStats
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_process_run_stats_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param ProcessRunStats body: Process stats (e.g. number of subjects processed) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_process_run_stats" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_process_run_stats`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/process-run-stats', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_project(self, body, **kwargs):
        """
        Report a new top-level Project
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_project(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Project body: Definition of project to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_project_with_http_info(body, **kwargs)
        else:
            (data) = self.report_project_with_http_info(body, **kwargs)
            return data

    def report_project_with_http_info(self, body, **kwargs):
        """
        Report a new top-level Project
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_project_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Project body: Definition of project to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_project`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/project', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_schema(self, body, **kwargs):
        """
        Report a new Schema
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_schema(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Schema body: A schema to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_schema_with_http_info(body, **kwargs)
        else:
            (data) = self.report_schema_with_http_info(body, **kwargs)
            return data

    def report_schema_with_http_info(self, body, **kwargs):
        """
        Report a new Schema
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_schema_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param Schema body: A schema to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_schema" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_schema`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/schema', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)

    def report_user(self, body, **kwargs):
        """
        Report a new User
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_user(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param User body: A User to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.report_user_with_http_info(body, **kwargs)
        else:
            (data) = self.report_user_with_http_info(body, **kwargs)
            return data

    def report_user_with_http_info(self, body, **kwargs):
        """
        Report a new User
        
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.report_user_with_http_info(body, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param User body: A User to be reported (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method report_user" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params) or (params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `report_user`")


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # Authentication setting
        auth_settings = []

        return self.api_client.call_api('/v1/dam/api/base-entities/user', 'POST',
                                        path_params,
                                        query_params,
                                        header_params,
                                        body=body_params,
                                        post_params=form_params,
                                        files=local_var_files,
                                        response_type=None,
                                        auth_settings=auth_settings,
                                        callback=params.get('callback'),
                                        _return_http_data_only=params.get('_return_http_data_only'),
                                        _preload_content=params.get('_preload_content', True),
                                        _request_timeout=params.get('_request_timeout'),
                                        collection_formats=collection_formats)


import pysftp
import os
import posixpath
import logging


# this import is actually used by end-user
from pysftp import CnOpts

from kensu.utils.helpers import report_simple_copy_with_guessed_schema


class Connection(pysftp.Connection):

    def __init__(self, host, username=None, private_key=None, password=None,
                 port=22, private_key_pass=None, ciphers=None, log=False,
                 cnopts=None, default_path=None):
        super().__init__(host,
                         username=username,
                         private_key=private_key,
                         password=password,
                         port=port,
                         private_key_pass=private_key_pass,
                         ciphers=ciphers,
                         log=log,
                         cnopts=cnopts,
                         default_path=default_path)
        self.ksu_sftp_host = host
        self.ksu_sftp_port = port

    def _ftp_server_addr(self):
        return f"ftp://{self.ksu_sftp_host}:{self.ksu_sftp_port}"

    def put(
            self,
            localpath,
            remotepath=None,
            callback=None,
            confirm=True,
            preserve_mtime=False
    ):
        result = super(Connection, self).put(
            localpath,
            remotepath=remotepath,
            callback=callback,
            confirm=confirm,
            preserve_mtime=preserve_mtime
        )
        if not remotepath:
            remotepath = os.path.split(localpath)[1]
        resolved_remote_path = posixpath.join(self.pwd, remotepath)
        qualified_remote_path = self._ftp_server_addr() + resolved_remote_path
        absolute_localpath = os.path.abspath(str(localpath))

        logging.info(f"sftp.put(localpath: {absolute_localpath}, remote_qualified_path:{qualified_remote_path})")
        # schema is not known for a generic ftp file copy !!! but we try to guess it
        from kensu.utils.helpers import get_absolute_path as get_kensu_abs_path
        kensu_normalized_absolute_path = get_kensu_abs_path(absolute_localpath)
        report_simple_copy_with_guessed_schema(
            input_uri=kensu_normalized_absolute_path,
            output_absolute_uri=qualified_remote_path,
            read_schema_from_filename=absolute_localpath,
            operation_type='sftp.put()'
        )
    put.__doc__ = pysftp.Connection.put.__doc__


Connection.__doc__ = pysftp.Connection.__doc__



class WithOutputBuilderElement(object):
    def __init__(self, builder):
        self.builder = builder

    def with_output(self, o):
        if self.builder.output is None:
            if isinstance(o, tuple):
                value = o[0]
                data_source = o[1]
                schema = o[2]
            else:
                value = o
                kensu = self.builder.kensu
                data_source = kensu.extractors.extract_data_source(value, kensu.default_physical_location_ref)
                schema = kensu.extractors.extract_schema(data_source, value)

            self.builder.output = value
            self.builder.output_data_source = data_source._report()
            self.builder.output_schema = schema._report()
            self.builder.output_fields_name = [f.name for f in self.builder.output_schema.pk.fields]

        from kensu.utils.dsl.add_deps_builder_element import AddDepsBuilderElement
        return AddDepsBuilderElement(self.builder)



import os
# disable Kensu collector when requested to do so (e.g. inside Apache Spark executor nodes)
if "KSU_DISABLE_PY_COLLECTOR" in os.environ:
    from pandas import *
else:
    import pandas as pd
    from pandas import *

    from .data_frame import DataFrame
    from .data_frame import Series
    from .data_frame import wrap_pandas_reader, wrap_pandas_get_dummies, wrap_merge, wrap_to_datetime, wrap_concat
    from .extractor import KensuPandasSupport

    if hasattr(pd, "read_clipboard"):
        read_clipboard = wrap_pandas_reader(pd.read_clipboard)
    if hasattr(pd, "read_csv"):
        read_csv = wrap_pandas_reader(pd.read_csv)
    if hasattr(pd, "read_excel"):
        read_excel = wrap_pandas_reader(pd.read_excel)
    if hasattr(pd, "read_feather"):
        read_feather = wrap_pandas_reader(pd.read_feather)
    if hasattr(pd, "read_fwf"):
        read_fwf = wrap_pandas_reader(pd.read_fwf)
    if hasattr(pd, "read_gbq"):
        read_gbq = wrap_pandas_reader(pd.read_gbq)
    if hasattr(pd, "read_hdf"):
        read_hdf = wrap_pandas_reader(pd.read_hdf)
    if hasattr(pd, "read_html"):
        read_html = wrap_pandas_reader(pd.read_html)
    if hasattr(pd, "read_json"):
        read_json = wrap_pandas_reader(pd.read_json)
    if hasattr(pd, "read_parquet"):
        read_parquet = wrap_pandas_reader(pd.read_parquet)
    if hasattr(pd, "read_pickle"):
        read_pickle = wrap_pandas_reader(pd.read_pickle)
    if hasattr(pd, "read_sas"):
        read_sas = wrap_pandas_reader(pd.read_sas)
    if hasattr(pd, "read_sql"):
        read_sql = wrap_pandas_reader(pd.read_sql)
    if hasattr(pd, "read_sql_query"):
        read_sql_query = wrap_pandas_reader(pd.read_sql_query)
    if hasattr(pd, "read_sql_table"):
        read_sql_table = wrap_pandas_reader(pd.read_sql_table)
    if hasattr(pd, "get_dummies"):
        get_dummies = wrap_pandas_get_dummies(pd.get_dummies)
    if hasattr(pd, "merge"):
        merge = wrap_merge(pd.merge)
    if hasattr(pd, 'to_datetime'):
        to_datetime = wrap_to_datetime(pd.to_datetime)
    if hasattr(pd, 'concat'):
        concat = wrap_concat(pd.concat)



# coding: utf-8

"""
    

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: beta
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import models into model package
from .batch_code_base import BatchCodeBase
from .batch_code_version import BatchCodeVersion
from .batch_data_source import BatchDataSource
from .batch_data_stats import BatchDataStats
from .batch_entity_report import BatchEntityReport
from .batch_entity_report_result import BatchEntityReportResult
from .batch_error_message import BatchErrorMessage
from .batch_lineage_run import BatchLineageRun
from .batch_model import BatchModel
from .batch_model_metrics import BatchModelMetrics
from .batch_model_training import BatchModelTraining
from .batch_physical_location import BatchPhysicalLocation
from .batch_process import BatchProcess
from .batch_process_lineage import BatchProcessLineage
from .batch_process_run import BatchProcessRun
from .batch_process_run_stats import BatchProcessRunStats
from .batch_project import BatchProject
from .batch_schema import BatchSchema
from .batch_schema_field_tag import BatchSchemaFieldTag
from .batch_user import BatchUser
from .code_base import CodeBase
from .code_base_pk import CodeBasePK
from .code_base_ref import CodeBaseRef
from .code_version import CodeVersion
from .code_version_pk import CodeVersionPK
from .code_version_ref import CodeVersionRef
from .data_source import DataSource
from .data_source_pk import DataSourcePK
from .data_source_ref import DataSourceRef
from .data_stats import DataStats
from .data_stats_pk import DataStatsPK
from .field_def import FieldDef
from .lineage_run import LineageRun
from .lineage_run_pk import LineageRunPK
from .lineage_run_ref import LineageRunRef
from .model import Model
from .model_metrics import ModelMetrics
from .model_metrics_pk import ModelMetricsPK
from .model_pk import ModelPK
from .model_ref import ModelRef
from .model_training import ModelTraining
from .model_training_pk import ModelTrainingPK
from .model_training_ref import ModelTrainingRef
from .physical_location import PhysicalLocation
from .physical_location_pk import PhysicalLocationPK
from .physical_location_ref import PhysicalLocationRef
from .process import Process
from .process_lineage import ProcessLineage
from .process_lineage_pk import ProcessLineagePK
from .process_lineage_ref import ProcessLineageRef
from .process_pk import ProcessPK
from .process_ref import ProcessRef
from .process_run import ProcessRun
from .process_run_pk import ProcessRunPK
from .process_run_ref import ProcessRunRef
from .process_run_stats import ProcessRunStats
from .process_run_stats_pk import ProcessRunStatsPK
from .project import Project
from .project_pk import ProjectPK
from .project_ref import ProjectRef
from .schema import Schema
from .schema_lineage_dependency_def import SchemaLineageDependencyDef
from .schema_pk import SchemaPK
from .schema_ref import SchemaRef
from .user import User
from .user_pk import UserPK
from .user_ref import UserRef